"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.herokuClientApiUrl = exports.APIError = void 0;
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
const url_1 = require("url");
const core_1 = require("@oclif/core");
const axios_1 = require("axios");
const axiosDebugger = require("axios-debug-log");
class APIError extends core_1.Errors.CLIError {
    constructor(httpError) {
        var _a;
        if (!httpError)
            throw new Error('invalid error');
        const options = (_a = httpError.response) === null || _a === void 0 ? void 0 : _a.data;
        if (!options || !options.message)
            throw httpError;
        const info = [];
        if (options.id)
            info.push(`Error ID: ${options.id}`);
        if (options.app && options.app.name)
            info.push(`App: ${options.app.name}`);
        if (options.url)
            info.push(`See ${options.url} for more information.`);
        if (info.length)
            super([options.message, ''].concat(info).join('\n'));
        else
            super(options.message);
        this.http = httpError;
        this.body = options;
    }
}
exports.APIError = APIError;
class APIClient {
    constructor(options) {
        this.auth = options.auth;
        this.apiUrl = options.apiUrl;
        const envHeaders = JSON.parse(process.env.SALESFORCE_FUNCTIONS_HEADERS || '{}');
        const opts = {
            baseURL: `${this.apiUrl.origin}`,
            headers: Object.assign({ Accept: 'application/vnd.heroku+json; version=3' }, envHeaders),
        };
        this.axios = axios_1.default.create(opts);
        axiosDebugger.addLogger(this.axios);
    }
    async request(url, options = {}) {
        options.headers = options.headers || {};
        if (!Object.keys(options.headers).find((header) => header.toLowerCase() === 'authorization')) {
            options.headers.authorization = `Bearer ${this.auth}`;
        }
        try {
            const response = await this.axios.request(Object.assign({ url }, options));
            return response;
        }
        catch (error) {
            if (!axios_1.default.isAxiosError(error)) {
                throw error;
            }
            throw new APIError(error);
        }
    }
    get(url, options = {}) {
        return this.request(url, Object.assign(Object.assign({}, options), { method: 'GET' }));
    }
    post(url, options = {}) {
        return this.request(url, Object.assign(Object.assign({}, options), { method: 'POST' }));
    }
    put(url, options = {}) {
        return this.request(url, Object.assign(Object.assign({}, options), { method: 'PUT' }));
    }
    patch(url, options = {}) {
        return this.request(url, Object.assign(Object.assign({}, options), { method: 'PATCH' }));
    }
    delete(url, options = {}) {
        return this.request(url, Object.assign(Object.assign({}, options), { method: 'DELETE' }));
    }
}
exports.default = APIClient;
function herokuClientApiUrl() {
    const defaultUrl = 'https://api.heroku.com';
    const envVarURL = process.env.SALESFORCE_FUNCTIONS_API;
    const apiURL = new url_1.URL(envVarURL || defaultUrl);
    return apiURL;
}
exports.herokuClientApiUrl = herokuClientApiUrl;
